#!/usr/bin/perl

use strict;
use warnings;
use utf8;
use File::Path;



###
### Const variable
###
my $OPT_DEBUG   = 0;
my $OPT_REMOVE  = 0;
my $OPT_FLAG    = 0;
my $OPT_HELP    = 0;
my $OPT_RETURN  = 1;
my @OPT_ARGS;

my $BIN_MD5SUM  = "/usr/bin/md5sum";
my $BIN_MKDIR   = "/usr/bin/mkdir";
my $BIN_WGET    = "/usr/bin/wget";
my $DATE        = `date "+%Y%m%d"`;
chop($DATE);

my $CIDR_ROOT_DIR   = "/var/iplist";
my $CIDR_FILE_DIR   = "$CIDR_ROOT_DIR/$DATE/RIR";

my %CIDR_FILE_URL;
$CIDR_FILE_URL{'ARIN'}      = "http://ftp.arin.net/pub/stats/arin/delegated-arin-extended-latest";
$CIDR_FILE_URL{'RIPE'}      = "http://ftp.ripe.net/pub/stats/ripencc/delegated-ripencc-extended-latest";
$CIDR_FILE_URL{'APNIC'}     = "http://ftp.apnic.net/pub/stats/apnic/delegated-apnic-extended-latest";
$CIDR_FILE_URL{'LACNIC'}    = "http://ftp.lacnic.net/pub/stats/lacnic/delegated-lacnic-extended-latest";
$CIDR_FILE_URL{'AFRINIC'}   = "http://ftp.afrinic.net/pub/stats/afrinic/delegated-afrinic-extended-latest";

my %CIDR_HASH_URL;
$CIDR_HASH_URL{'ARIN'}      = "http://ftp.arin.net/pub/stats/arin/delegated-arin-extended-latest.md5";
$CIDR_HASH_URL{'RIPE'}      = "http://ftp.ripe.net/pub/stats/ripencc/delegated-ripencc-extended-latest.md5";
$CIDR_HASH_URL{'APNIC'}     = "http://ftp.apnic.net/stats/apnic/delegated-apnic-extended-latest.md5";
$CIDR_HASH_URL{'LACNIC'}    = "http://ftp.lacnic.net/pub/stats/lacnic/delegated-lacnic-extended-latest.md5";
$CIDR_HASH_URL{'AFRINIC'}   = "http://ftp.afrinic.net/pub/stats/afrinic/delegated-afrinic-extended-latest.md5";



###
### Sobroutine
###
## Get iplist from RIR
##
sub GET_DELEGATE_IPLIST{
    # Get RIR cidr iplist
    foreach my $tmp_FILE(sort keys(%CIDR_FILE_URL)){
        if($OPT_DEBUG) { print "VALUE::CIDR_FILE_URL $CIDR_FILE_URL{$tmp_FILE}\n"; }
        my $tmp_RESULT = system("$BIN_WGET -t 10 -q  $CIDR_FILE_URL{$tmp_FILE} -O $CIDR_FILE_DIR/$tmp_FILE.IP");
        if($tmp_RESULT){ return 1; }
    }
    if($OPT_DEBUG){ print "-----\n"; }

    # Get RIR cidr iplist hash sum
    foreach my $tmp_FILE(sort keys(%CIDR_HASH_URL)){
        if($OPT_DEBUG) { print "VALUE::CIDR_HASH_URL $CIDR_HASH_URL{$tmp_FILE}\n"; }
        my $tmp_RESULT = system("$BIN_WGET -t 10 -q  $CIDR_HASH_URL{$tmp_FILE} -O $CIDR_FILE_DIR/$tmp_FILE.MD5");
        if($tmp_RESULT){ return 1; }
    }
    if($OPT_DEBUG){ print "-----\n"; }
    return 0;
}


## Check hash of download ip list
##
sub CHECK_FILE_HASH{
    foreach my $tmp_FILE(sort keys(%CIDR_FILE_URL)){
        my $HASH_FROM;
        my $HASH_TO;
        my $tmp_FLAG   = 1;
        my $tmp_RESULT = `$BIN_MD5SUM $CIDR_FILE_DIR/$tmp_FILE.IP`;
        my @tmp_HASH   = split(/\s+/, $tmp_RESULT);

        # Open md5 file and check sum
        if(-f "$CIDR_FILE_DIR/$tmp_FILE.MD5"){
            open(READ_MD5, "$CIDR_FILE_DIR/$tmp_FILE.MD5");
            foreach(<READ_MD5>){
                foreach(split(/\s+/)){

                    # File hash formatting and comparison
                    if((length($_) == 32)&&($_ !~ /[^a-zA-Z0-9]/)){
                        $HASH_FROM = lc($tmp_HASH[0]);
                        $HASH_TO   = lc($_);
                        if($OPT_DEBUG){
                            print "FILE::IPLIST $tmp_FILE\n";
                            print "HASH::FROM   $HASH_FROM\n";
                            print "HASH::TO     $HASH_TO\n";
                            print "-----\n";
                        }
                        unless("$HASH_FROM" cmp "$HASH_TO"){ $tmp_FLAG = 0; }
                    }
                }
            }
            close(READ_MD5);
        }
        if($tmp_FLAG){ return 1; }
    }
    return 0;
}


## Output help
##
sub OUTPUT_SCRIPT_HELP{
    print 'Usage: make-country-getrir.pl -s OBJECT [ OPTIONS ]'."\n";
    print '                                                   '."\n";
    print 'OBJECT  := ROOTPATH     / Save in root directory   '."\n";
    print 'OPTIONS := -d[--debug]  / Standard stream for debug'."\n";
    print '           -h[--help]   / Output of this help      '."\n";
    print '           -r[--remove] / Format ip list directory '."\n";
    print '                                                   '."\n";
    print 'Example of execution:                              '."\n";
    print '  $ ./make-country-getrir.pl                       '."\n";
    print '  $ ./make-country-getrir.pl -s ROOTPATH           '."\n";
    print '  $ ./make-country-getrir.pl -s ROOTPATH --remove  '."\n";
    return 0;
}



###
### Main Program
###
## Setting options and arguments
##
# Set of execution options
foreach my $tmp_OPT(@ARGV){
    push(@OPT_ARGS, $tmp_OPT);
    if(($tmp_OPT eq '-d')||($tmp_OPT eq '--debug' )){ $OPT_DEBUG++;  }
    if(($tmp_OPT eq '-h')||($tmp_OPT eq '--help'  )){ $OPT_HELP++;   }
    if(($tmp_OPT eq '-r')||($tmp_OPT eq '--remove')){ $OPT_REMOVE++; }
}
if($OPT_DEBUG){
    print "VALUE::OPT_DEBUG  $OPT_DEBUG\n";
    print "VALUE::OPT_HELP   $OPT_HELP\n";
    print "VALUE::OPT_REMOVE $OPT_REMOVE\n";
    print "-----\n";
}

# Set of ip list save root directory
for(my $tmp_COUNT=0; $tmp_COUNT<($#OPT_ARGS+1); $tmp_COUNT++){
    if($OPT_ARGS[$tmp_COUNT] eq '-s'){
        $CIDR_ROOT_DIR = $OPT_ARGS[$tmp_COUNT+1];
        $CIDR_FILE_DIR = "$CIDR_ROOT_DIR/$DATE/RIR";
    }
}
if($OPT_DEBUG){
    print "VALUE::CIDR_ROOT_DIR $CIDR_ROOT_DIR\n";
    print "VALUE::CIDR_FILE_DIR $CIDR_FILE_DIR\n";
    print "-----\n";
}

# Execute of initial process or help
if($OPT_HELP){
    $OPT_RETURN = &OUTPUT_SCRIPT_HELP;
}else{
    if($OPT_REMOVE){
        if(-d "$CIDR_FILE_DIR"){ rmtree($CIDR_FILE_DIR); }
    }
    unless(-d "$CIDR_FILE_DIR"){ system("$BIN_MKDIR -p $CIDR_FILE_DIR"); }
    unless($OPT_FLAG){ $OPT_FLAG = &GET_DELEGATE_IPLIST; }
    unless($OPT_FLAG){ $OPT_FLAG = &CHECK_FILE_HASH;     }
    unless($OPT_FLAG){ $OPT_RETURN = 0; }
}

exit $OPT_RETURN;